@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,11 +14,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import datetime
 import uuid
 
 import mock
-from oslo_utils import timeutils
 
 from gnocchi import indexer
 from gnocchi import statsd
@@ -52,11 +51,11 @@ class TestStatsd(tests_base.TestCase):
     def test_flush_empty(self):
         self.server.stats.flush()
 
-    @mock.patch.object(timeutils, 'utcnow')
+    @mock.patch.object(utils, 'utcnow')
     def _test_gauge_or_ms(self, metric_type, utcnow):
         metric_name = "test_gauge_or_ms"
         metric_key = metric_name + "|" + metric_type
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 58, 36)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 58, 36)
         self.server.datagram_received(
             ("%s:1|%s" % (metric_name, metric_type)).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -69,7 +68,7 @@ class TestStatsd(tests_base.TestCase):
         metric = r.get_metric(metric_key)
 
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -78,7 +77,7 @@ class TestStatsd(tests_base.TestCase):
             (utils.datetime_utc(2015, 1, 7, 13, 58), 60.0, 1.0)
         ], measures)
 
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 59, 37)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 59, 37)
         # This one is going to be ignored
         self.server.datagram_received(
             ("%s:45|%s" % (metric_name, metric_type)).encode('ascii'),
@@ -89,7 +88,7 @@ class TestStatsd(tests_base.TestCase):
         self.stats.flush()
 
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -105,11 +104,11 @@ class TestStatsd(tests_base.TestCase):
     def test_ms(self):
         self._test_gauge_or_ms("ms")
 
-    @mock.patch.object(timeutils, 'utcnow')
+    @mock.patch.object(utils, 'utcnow')
     def test_counter(self, utcnow):
         metric_name = "test_counter"
         metric_key = metric_name + "|c"
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 58, 36)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 58, 36)
         self.server.datagram_received(
             ("%s:1|c" % metric_name).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -122,7 +121,7 @@ class TestStatsd(tests_base.TestCase):
         self.assertIsNotNone(metric)
 
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
@@ -130,7 +129,7 @@ class TestStatsd(tests_base.TestCase):
             (utils.datetime_utc(2015, 1, 7, 13), 3600.0, 1.0),
             (utils.datetime_utc(2015, 1, 7, 13, 58), 60.0, 1.0)], measures)
 
-        utcnow.return_value = datetime.datetime(2015, 1, 7, 13, 59, 37)
+        utcnow.return_value = utils.datetime_utc(2015, 1, 7, 13, 59, 37)
         self.server.datagram_received(
             ("%s:45|c" % metric_name).encode('ascii'),
             ("127.0.0.1", 12345))
@@ -140,7 +139,7 @@ class TestStatsd(tests_base.TestCase):
         self.stats.flush()
 
         self.stats.storage.process_background_tasks(
-            self.stats.indexer, sync=True)
+            self.stats.indexer, [str(metric.id)], sync=True)
 
         measures = self.stats.storage.get_measures(metric)
         self.assertEqual([
