@@ -21,9 +21,11 @@ from oslo_utils import timeutils
 from oslotest import base
 import six.moves
 
+from gnocchi import archive_policy
+from gnocchi import carbonara
+from gnocchi import indexer
 from gnocchi import storage
 from gnocchi.storage import _carbonara
-from gnocchi.storage import null
 from gnocchi.tests import base as tests_base
 from gnocchi import utils
 
@@ -42,50 +44,78 @@ class TestStorageDriver(tests_base.TestCase):
                                          archive_policy_name)
         return m, m_sql
 
-    def test_get_driver(self):
-        self.conf.set_override('driver', 'null', 'storage')
-        driver = storage.get_driver(self.conf)
-        self.assertIsInstance(driver, null.NullStorage)
+    def trigger_processing(self, metrics=None):
+        if metrics is None:
+            metrics = [str(self.metric.id)]
+        self.storage.process_background_tasks(self.index, metrics, sync=True)
 
-    @mock.patch('gnocchi.storage._carbonara.LOG')
-    def test_corrupted_data(self, logger):
+    def test_get_driver(self):
+        driver = storage.get_driver(self.conf)
+        self.assertIsInstance(driver, storage.StorageDriver)
+
+    def test_corrupted_data(self):
         if not isinstance(self.storage, _carbonara.CarbonaraBasedStorage):
             self.skipTest("This driver is not based on Carbonara")
 
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
 
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 13, 0, 1), 1),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 13, 0, 1), 1),
         ])
-        with mock.patch('gnocchi.carbonara.msgpack.unpack',
-                        side_effect=ValueError("boom!")):
-            with mock.patch('gnocchi.carbonara.msgpack.loads',
-                            side_effect=ValueError("boom!")):
-                self.storage.process_background_tasks(self.index, sync=True)
 
-        self.assertEqual([
-            (utils.datetime_utc(2014, 1, 1), 86400.0, 1),
-            (utils.datetime_utc(2014, 1, 1, 13), 3600.0, 1),
-            (utils.datetime_utc(2014, 1, 1, 13), 300.0, 1),
-        ], self.storage.get_measures(self.metric))
+        with mock.patch('gnocchi.carbonara.AggregatedTimeSerie.unserialize',
+                        side_effect=ValueError("boom!")):
+            with mock.patch('gnocchi.carbonara.BoundTimeSerie.unserialize',
+                            side_effect=ValueError("boom!")):
+                self.trigger_processing()
+
+        m = self.storage.get_measures(self.metric)
+        self.assertIn((utils.datetime_utc(2014, 1, 1), 86400.0, 1), m)
+        self.assertIn((utils.datetime_utc(2014, 1, 1, 13), 3600.0, 1), m)
+        self.assertIn((utils.datetime_utc(2014, 1, 1, 13), 300.0, 1), m)
+
+    def test_list_metric_with_measures_to_process(self):
+        metrics = self.storage.list_metric_with_measures_to_process(
+            None, None, full=True)
+        self.assertEqual(set(), metrics)
+        self.storage.add_measures(self.metric, [
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        ])
+        metrics = self.storage.list_metric_with_measures_to_process(
+            None, None, full=True)
+        self.assertEqual(set([str(self.metric.id)]), metrics)
+        self.trigger_processing()
+        metrics = self.storage.list_metric_with_measures_to_process(
+            None, None, full=True)
+        self.assertEqual(set([]), metrics)
 
     def test_delete_nonempty_metric(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
-        self.storage.delete_metric(self.metric)
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
+        self.storage.delete_metric(self.metric, sync=True)
+        self.trigger_processing()
 
     def test_delete_nonempty_metric_unprocessed(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
         ])
-        self.storage.delete_metric(self.metric)
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.storage.delete_metric(self.metric, sync=True)
+        self.trigger_processing()
+
+    def test_delete_expunge_metric(self):
+        self.storage.add_measures(self.metric, [
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+        ])
+        self.trigger_processing()
+        self.index.delete_metric(self.metric.id)
+        self.storage.expunge_metrics(self.index, sync=True)
+        self.assertRaises(indexer.NoSuchMetric, self.index.delete_metric,
+                          self.metric.id)
 
     def test_measures_reporting(self):
         report = self.storage.measures_report(True)
@@ -105,42 +135,63 @@ class TestStorageDriver(tests_base.TestCase):
     def test_add_measures_big(self):
         m, __ = self._create_metric('high')
         self.storage.add_measures(m, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, i, j), 100)
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, i, j), 100)
             for i in six.moves.range(0, 60) for j in six.moves.range(0, 60)])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing([str(m.id)])
 
         self.assertEqual(3661, len(self.storage.get_measures(m)))
 
-    @mock.patch('gnocchi.carbonara.AggregatedTimeSerie.POINTS_PER_SPLIT', 48)
-    def test_add_measures_big_update_subset(self):
+    @mock.patch('gnocchi.carbonara.SplitKey.POINTS_PER_SPLIT', 48)
+    def test_add_measures_update_subset_split(self):
         m, m_sql = self._create_metric('medium')
         measures = [
-            storage.Measure(datetime.datetime(2014, 1, i, j, 0, 0), 100)
-            for i in six.moves.range(1, 6) for j in six.moves.range(0, 24)]
-        measures.append(
-            storage.Measure(datetime.datetime(2014, 1, 6, 0, 0, 0), 100))
+            storage.Measure(utils.datetime_utc(2014, 1, 6, i, j, 0), 100)
+            for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
         self.storage.add_measures(m, measures)
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing([str(m.id)])
 
+        # add measure to end, in same aggregate time as last point.
         self.storage.add_measures(m, [
-            storage.Measure(datetime.datetime(2014, 1, 6, 1, 0, 0), 100)])
+            storage.Measure(utils.datetime_utc(2014, 1, 6, 1, 58, 1), 100)])
 
         with mock.patch.object(self.storage, '_store_metric_measures') as c:
-            self.storage.process_background_tasks(self.index, sync=True)
+            # should only resample last aggregate
+            self.trigger_processing([str(m.id)])
         count = 0
         for call in c.mock_calls:
-            if mock.call(m_sql, mock.ANY, 'mean', 3600.0, mock.ANY) == call:
+            # policy is 60 points and split is 48. should only update 2nd half
+            args = call[1]
+            if args[0] == m_sql and args[2] == 'mean' and args[3] == 60.0:
                 count += 1
         self.assertEqual(1, count)
 
+    def test_add_measures_update_subset(self):
+        m, m_sql = self._create_metric('medium')
+        measures = [
+            storage.Measure(utils.datetime_utc(2014, 1, 6, i, j, 0), 100)
+            for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
+        self.storage.add_measures(m, measures)
+        self.trigger_processing([str(m.id)])
+
+        # add measure to end, in same aggregate time as last point.
+        new_point = utils.datetime_utc(2014, 1, 6, 1, 58, 1)
+        self.storage.add_measures(m, [storage.Measure(new_point, 100)])
+
+        with mock.patch.object(self.storage, '_add_measures') as c:
+            self.trigger_processing([str(m.id)])
+        for __, args, __ in c.mock_calls:
+            self.assertEqual(
+                list(args[3])[0][0], carbonara.round_timestamp(
+                    new_point, args[1].granularity * 10e8))
+
     def test_delete_old_measures(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
@@ -152,9 +203,9 @@ class TestStorageDriver(tests_base.TestCase):
 
         # One year later…
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2015, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2015, 1, 1, 12, 0, 1), 69),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
@@ -163,18 +214,121 @@ class TestStorageDriver(tests_base.TestCase):
             (utils.datetime_utc(2015, 1, 1, 12), 300.0, 69),
         ], self.storage.get_measures(self.metric))
 
+        self.assertEqual({"1244160000.0"},
+                         self.storage._list_split_keys_for_metric(
+                             self.metric, "mean", 86400.0))
+        self.assertEqual({"1412640000.0"},
+                         self.storage._list_split_keys_for_metric(
+                             self.metric, "mean", 3600.0))
+        self.assertEqual({"1419120000.0"},
+                         self.storage._list_split_keys_for_metric(
+                             self.metric, "mean", 300.0))
+
+    def test_rewrite_measures(self):
+        # Create an archive policy that spans on several splits. Each split
+        # being 3600 points, let's go for 36k points so we have 10 splits.
+        apname = str(uuid.uuid4())
+        ap = archive_policy.ArchivePolicy(apname, 0, [(36000, 60)])
+        self.index.create_archive_policy(ap)
+        self.metric = storage.Metric(uuid.uuid4(), ap)
+        self.index.create_metric(self.metric.id, str(uuid.uuid4()),
+                                 str(uuid.uuid4()),
+                                 apname)
+
+        # First store some points scattered across different splits
+        self.storage.add_measures(self.metric, [
+            storage.Measure(utils.datetime_utc(2016, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2016, 1, 2, 13, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2016, 1, 4, 14, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2016, 1, 6, 15, 12, 45), 44),
+        ])
+        self.trigger_processing()
+
+        splits = {'1451520000.0', '1451736000.0', '1451952000.0'}
+        self.assertEqual(splits,
+                         self.storage._list_split_keys_for_metric(
+                             self.metric, "mean", 60.0))
+
+        if self.storage.WRITE_FULL:
+            assertCompressedIfWriteFull = self.assertTrue
+        else:
+            assertCompressedIfWriteFull = self.assertFalse
+
+        data = self.storage._get_measures(
+            self.metric, '1451520000.0', "mean", 60.0)
+        self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
+        data = self.storage._get_measures(
+            self.metric, '1451736000.0', "mean", 60.0)
+        self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
+        data = self.storage._get_measures(
+            self.metric, '1451952000.0', "mean", 60.0)
+        assertCompressedIfWriteFull(
+            carbonara.AggregatedTimeSerie.is_compressed(data))
+
+        self.assertEqual([
+            (utils.datetime_utc(2016, 1, 1, 12), 60.0, 69),
+            (utils.datetime_utc(2016, 1, 2, 13, 7), 60.0, 42),
+            (utils.datetime_utc(2016, 1, 4, 14, 9), 60.0, 4),
+            (utils.datetime_utc(2016, 1, 6, 15, 12), 60.0, 44),
+        ], self.storage.get_measures(self.metric, granularity=60.0))
+
+        # Now store brand new points that should force a rewrite of one of the
+        # split (keep in mind the back window size in one hour here). We move
+        # the BoundTimeSerie processing timeserie far away from its current
+        # range.
+        self.storage.add_measures(self.metric, [
+            storage.Measure(utils.datetime_utc(2016, 1, 10, 16, 18, 45), 45),
+            storage.Measure(utils.datetime_utc(2016, 1, 10, 17, 12, 45), 46),
+        ])
+        self.trigger_processing()
+
+        self.assertEqual({'1452384000.0', '1451736000.0',
+                          '1451520000.0', '1451952000.0'},
+                         self.storage._list_split_keys_for_metric(
+                             self.metric, "mean", 60.0))
+        data = self.storage._get_measures(
+            self.metric, '1451520000.0', "mean", 60.0)
+        self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
+        data = self.storage._get_measures(
+            self.metric, '1451736000.0', "mean", 60.0)
+        self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
+        data = self.storage._get_measures(
+            self.metric, '1451952000.0', "mean", 60.0)
+        # Now this one is compressed because it has been rewritten!
+        self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
+        data = self.storage._get_measures(
+            self.metric, '1452384000.0', "mean", 60.0)
+        assertCompressedIfWriteFull(
+            carbonara.AggregatedTimeSerie.is_compressed(data))
+
+        self.assertEqual([
+            (utils.datetime_utc(2016, 1, 1, 12), 60.0, 69),
+            (utils.datetime_utc(2016, 1, 2, 13, 7), 60.0, 42),
+            (utils.datetime_utc(2016, 1, 4, 14, 9), 60.0, 4),
+            (utils.datetime_utc(2016, 1, 6, 15, 12), 60.0, 44),
+            (utils.datetime_utc(2016, 1, 10, 16, 18), 60.0, 45),
+            (utils.datetime_utc(2016, 1, 10, 17, 12), 60.0, 46),
+        ], self.storage.get_measures(self.metric, granularity=60.0))
+
     def test_updated_measures(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
+
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 55.5),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 55.5),
+            (utils.datetime_utc(2014, 1, 1, 12), 300.0, 69),
+            (utils.datetime_utc(2014, 1, 1, 12, 5), 300.0, 42.0),
+        ], self.storage.get_measures(self.metric))
 
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
@@ -202,12 +356,12 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_add_and_get_measures(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing()
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
@@ -292,10 +446,10 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_get_measure_unknown_aggregation(self):
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.AggregationDoesNotExist,
                           self.storage.get_measures,
@@ -305,16 +459,16 @@ class TestStorageDriver(tests_base.TestCase):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['low'])
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.AggregationDoesNotExist,
                           self.storage.get_cross_metric_measures,
@@ -325,16 +479,16 @@ class TestStorageDriver(tests_base.TestCase):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['low'])
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.assertRaises(storage.GranularityDoesNotExist,
                           self.storage.get_cross_metric_measures,
@@ -345,16 +499,16 @@ class TestStorageDriver(tests_base.TestCase):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['no_granularity_match'])
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
 
         self.assertRaises(storage.MetricUnaggregatable,
@@ -364,18 +518,18 @@ class TestStorageDriver(tests_base.TestCase):
     def test_add_and_get_cross_metric_measures(self):
         metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 44),
         ])
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 41), 2),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 10, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 41), 2),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 10, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 4),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
         values = self.storage.get_cross_metric_measures([self.metric, metric2])
         self.assertEqual([
@@ -386,6 +540,16 @@ class TestStorageDriver(tests_base.TestCase):
             (utils.datetime_utc(2014, 1, 1, 12, 10, 0), 300.0, 24.0)
         ], values)
 
+        values = self.storage.get_cross_metric_measures([self.metric, metric2],
+                                                        reaggregation='max')
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1, 0, 0, 0), 86400.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 3600.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 300.0, 69),
+            (utils.datetime_utc(2014, 1, 1, 12, 5, 0), 300.0, 23),
+            (utils.datetime_utc(2014, 1, 1, 12, 10, 0), 300.0, 44)
+        ], values)
+
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
             from_timestamp=utils.to_timestamp('2014-01-01 12:10:00'))
@@ -439,19 +603,19 @@ class TestStorageDriver(tests_base.TestCase):
     def test_add_and_get_cross_metric_measures_with_holes(self):
         metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 5, 31), 8),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 5, 31), 8),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 42),
         ])
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 2),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 6),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 2),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 2),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 6),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 2),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
         values = self.storage.get_cross_metric_measures([self.metric, metric2])
         self.assertEqual([
@@ -465,20 +629,20 @@ class TestStorageDriver(tests_base.TestCase):
     def test_search_value(self):
         metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1,), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 5, 31), 8),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 1,), 69),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 5, 31), 8),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 12, 45), 42),
         ])
 
         self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 5), 9),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 2),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 6),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 2),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 9),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 7, 31), 2),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 9, 31), 6),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 13, 10), 2),
         ])
-        self.storage.process_background_tasks(self.index, sync=True)
+        self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
         self.assertEqual(
             {metric2: [],
@@ -499,6 +663,47 @@ class TestStorageDriver(tests_base.TestCase):
                     {u"eq": 100},
                     {u"≠": 50}]}))
 
+    def test_resize_policy(self):
+        name = str(uuid.uuid4())
+        ap = archive_policy.ArchivePolicy(name, 0, [(3, 5)])
+        self.index.create_archive_policy(ap)
+        m = self.index.create_metric(uuid.uuid4(), str(uuid.uuid4()),
+                                     str(uuid.uuid4()), name)
+        m = self.index.list_metrics(ids=[m.id])[0]
+        self.storage.add_measures(m, [
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 0), 1),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 5), 1),
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 10), 1),
+        ])
+        self.trigger_processing([str(m.id)])
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 5), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 10), 5.0, 1.0),
+        ], self.storage.get_measures(m))
+        # expand to more points
+        self.index.update_archive_policy(
+            name, [archive_policy.ArchivePolicyItem(granularity=5, points=6)])
+        m = self.index.list_metrics(ids=[m.id])[0]
+        self.storage.add_measures(m, [
+            storage.Measure(utils.datetime_utc(2014, 1, 1, 12, 0, 15), 1),
+        ])
+        self.trigger_processing([str(m.id)])
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 5), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 10), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 15), 5.0, 1.0),
+        ], self.storage.get_measures(m))
+        # shrink timespan
+        self.index.update_archive_policy(
+            name, [archive_policy.ArchivePolicyItem(granularity=5, points=2)])
+        m = self.index.list_metrics(ids=[m.id])[0]
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 10), 5.0, 1.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 15), 5.0, 1.0),
+        ], self.storage.get_measures(m))
+
 
 class TestMeasureQuery(base.BaseTestCase):
     def test_equal(self):
